import { PluginObj } from '@babel/core';
import { TemplateBuilder } from '@babel/template';
import { TSExportAssignment } from '@babel/types';
declare const _default: ({ template }: {
    template: TemplateBuilder<TSExportAssignment>;
}) => PluginObj<{}>;
export = _default;
//# sourceMappingURL=index.d.ts.map