import { format, Incident } from "./incident";

export { format, Incident } from "./incident";
export { StaticIncident } from "./types";

const DEFAULT: any = {format, Incident};

export { DEFAULT as default };
